/*
 *
 * CDDL HEADER START
 *
 * The contents of this file are subject to the terms of the
 * Common Development and Distribution License, Version 1.0 only
 * (the "License"). You may not use this file except in compliance
 * with the License.
 *
 * You can obtain a copy of the license at license/ESCIDOC.LICENSE
 * or http://www.escidoc.de/license.
 * See the License for the specific language governing permissions
 * and limitations under the License.
 *
 * When distributing Covered Code, include this CDDL HEADER in each
 * file and include the License file at license/ESCIDOC.LICENSE.
 * If applicable, add the following below this CDDL HEADER, with the
 * fields enclosed by brackets "[]" replaced with your own identifying
 * information: Portions Copyright [yyyy] [name of copyright owner]
 *
 * CDDL HEADER END
 */
/*
 * Copyright 2006-2007 Fachinformationszentrum Karlsruhe Gesellschaft
 * für wissenschaftlich-technische Information mbH and Max-Planck-
 * Gesellschaft zur Förderung der Wissenschaft e.V.
 * All rights reserved. Use is subject to license terms.
 */
package de.mpg.imeji.logic.export.format.explain;

import java.io.OutputStream;
import java.io.PrintWriter;
import java.net.URI;

import de.mpg.imeji.logic.export.format.ExplainExport;
import de.mpg.imeji.logic.search.SearchResult;
import de.mpg.imeji.logic.search.vo.SearchIndex;
import de.mpg.imeji.logic.vo.CollectionImeji;
import de.mpg.imeji.logic.vo.Statement;
import de.mpg.imeji.presentation.search.URLQueryTransformer;
import de.mpg.imeji.presentation.util.ObjectCachedLoader;

/**
 * {@link ExplainExport} for the metadata search index
 * 
 * @author saquet (initial creation)
 * @author $Author$ (last modification)
 * @version $Revision$ $LastChangedDate$
 */
public class MetadataExplainExport extends ExplainExport
{
    /*
     * (non-Javadoc)
     * @see de.mpg.imeji.logic.export.Export#export(java.io.OutputStream, de.mpg.imeji.logic.search.SearchResult)
     */
    @Override
    public void export(OutputStream out, SearchResult sr)
    {
        PrintWriter writer = new PrintWriter(out);
        writer.append(getRDFTagOpen());
        for (String colURI : sr.getResults())
        {
            CollectionImeji col = ObjectCachedLoader.loadCollection(URI.create(colURI));
            for (Statement st : ObjectCachedLoader.loadProfile(col.getProfile()).getStatements())
            {
                for (SearchIndex index : SearchIndex.getAllIndexForStatement(st))
                {
                    writer.append(getIndexTag(URLQueryTransformer.transformStatementToIndex(st.getId(), index),
                            index.getNamespace()));
                }
            }
        }
        writer.append(getRDFTagClose());
        writer.close();
    }

    /*
     * (non-Javadoc)
     * @see de.mpg.imeji.logic.export.Export#init()
     */
    @Override
    public void init()
    {
        // Nothing to to
    }
}
